// Simox
#include <SimoxUtility/algorithm/vector.hpp>

// BaseClass
#include "PKFinder.h"

// ArmarX

namespace armarx::priorknowledge::core
{
    void
    PKFinder::registerPropertyDefinitions(armarx::PropertyDefinitionsPtr& defs,
                                          const std::string& prefix)
    {
        if (not defs->hasDefinition(prefix + "packageName"))
        {
            defs->optional(this->packageName,
                           prefix + "packageName",
                           "The name of the prior knowledge data package.");
        }
    }

    void
    PKFinder::init()
    {
        this->recalculateBasePath();
    }

    bool
    PKFinder::checkAll() const
    {
        const std::filesystem::path absPath = Base::getFullPath();
        if (std::filesystem::is_regular_file(absPath))
        {
            ARMARX_WARNING << "The entered path is leading to a file!";
            return false;
        }

        for (const auto& d : std::filesystem::directory_iterator(absPath))
        {
            std::string k = d.path().filename();
            if (simox::alg::contains(ID_BLACKLIST, k))
            {
                continue;
            }

            if (not this->check(d.path()))
            {
                return false;
            }
        }
        return true;
    }

    bool
    PKFinder::check(const std::string& id) const
    {
        if (simox::alg::contains(ID_BLACKLIST, id))
        {
            return false;
        }

        const std::filesystem::path absPath = Base::getFullPath() / id;

        return this->accept(absPath);
    }

    std::optional<PKFinderInfo>
    PKFinder::find(const std::string& id) const
    {
        const std::filesystem::path absPath = Base::getFullPath() / id;

        if (this->check(absPath))
        {
            return PKFinderInfo{
                getPackageName(), getAbsolutePackagePath(), getRelativePath() / id, id};
        }
        return std::nullopt;
    }

    std::vector<PKFinderInfo>
    PKFinder::findAll() const
    {
        const std::filesystem::path absPath = Base::getFullPath();
        if (std::filesystem::is_regular_file(absPath))
        {
            ARMARX_WARNING << "The entered path is leading to a file!";
            return {};
        }

        std::vector<PKFinderInfo> ret;
        for (const auto& d : std::filesystem::directory_iterator(absPath))
        {
            std::string k = d.path().filename();
            if (simox::alg::contains(ID_BLACKLIST, k))
            {
                continue;
            }

            auto el = this->find(k);
            if (el.has_value())
            {
                ret.push_back(*el);
            }
        }
        return ret;
    }

} // namespace armarx::priorknowledge::core
