/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::armem
 * @author     Simon Ottenhaus ( simon dot ottenhaus at kit dot edu )
 * @date       2020
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#define BOOST_TEST_MODULE RobotAPI::ArmarXLibraries::armem

#define ARMARX_BOOST_TEST

#include <iostream>
#include <set>

#include <RobotAPI/Test.h>
#include <RobotAPI/libraries/armem/core/wm/memory_definitions.h>
#include <RobotAPI/libraries/aron/common/aron/Color.aron.generated.h>
#include <RobotAPI/libraries/aron/common/aron/time.aron.generated.h>

BOOST_AUTO_TEST_CASE(test_EntityInstance_withDataAs_Color)
{
    simox::arondto::Color color;
    color.r = 0;
    color.g = 100;
    color.b = 200;
    color.a = 255;

    armarx::armem::wm::EntityInstance instance;
    instance.data() = color.toAron();

    const armarx::armem::wm::EntityInstanceBase<simox::arondto::Color> w =
        instance.withDataAs<simox::arondto::Color>();
    const simox::arondto::Color& colorOut = w.data();

    BOOST_CHECK(colorOut == color);
    BOOST_CHECK_EQUAL(colorOut.r, 0);
    BOOST_CHECK_EQUAL(colorOut.g, 100);
    BOOST_CHECK_EQUAL(colorOut.b, 200);
    BOOST_CHECK_EQUAL(colorOut.a, 255);
}

BOOST_AUTO_TEST_CASE(test_EntityInstance_withDataAs_Time)
{
    armarx::arondto::Duration duration;
    duration.microSeconds = 1000;

    armarx::armem::wm::EntityInstance instance;
    instance.data() = duration.toAron();

    const armarx::armem::wm::EntityInstanceBase<armarx::arondto::Duration> w =
        instance.withDataAs<armarx::arondto::Duration>();
    const armarx::arondto::Duration& durationOut = w.data();

    BOOST_CHECK(durationOut == duration);
    BOOST_CHECK_EQUAL(durationOut.microSeconds, 1000);
    assert(durationOut.microSeconds == 1000);
}
